@@ -13,6 +13,8 @@ import Queue
 import socket
 import tldextract
 tldextract.tldextract.LOG.setLevel(logging.WARNING)
+import difflib
+
 
 from pilot.api.api import PilotError
 
@@ -53,7 +55,7 @@ except:
 #from pilot.coordination.advert import AdvertCoordinationAdaptor as CoordinationAdaptor
 #from pilot.coordination.nocoord import NoCoordinationAdaptor as CoordinationAdaptor
 from pilot.coordination.redis_adaptor import RedisCoordinationAdaptor as CoordinationAdaptor
-from bliss.saga import Url as SAGAUrl
+from saga import Url as SAGAUrl
 
 
 # generate global application id for this instance
@@ -240,7 +242,8 @@ class PilotData(PilotData):
         
         if self.pilot_data_description!=None:
             self.service_url=self.pilot_data_description["service_url"]
-            self.size = self.pilot_data_description["size"]
+            if self.pilot_data_description.has_key("size"):
+                self.size = self.pilot_data_description["size"]
             
             # initialize file adaptor
             if self.service_url.startswith("ssh:"):
@@ -589,7 +592,6 @@ class DataUnit(DataUnit):
         """ add this DU (self) to a certain pilot data 
             data will be moved into this data
         """
-        self.state=State.Pending
         transfer_thread=threading.Thread(target=self.__add_pilot_data, args=[pilot_data])
         transfer_thread.start()        
         self.transfer_threads.append(transfer_thread)
@@ -610,14 +612,25 @@ class DataUnit(DataUnit):
         
         if len(self.pilot_data) > 0:
             # Search for PD that is close to local machine
+            local_hostname=socket.getfqdn()
+            max_score=0
+            best_pd=None
             for pd in self.pilot_data:
-                pd_domain = tldextract.extract(pd.service_url).domain
-                local_domain = tldextract.extract(socket.getfqdn()).domain
-                logger.debug("Export to %s... checking PD at: %s"%(local_domain, pd_domain))
-                if pd_domain == local_domain:
-                    logger.debug("Export from: %s"%(pd_domain))
-                    pd.export_du(self, target_url)
-                    return
+                pd_host = SAGAUrl(pd.service_url).host
+                pd_score = difflib.SequenceMatcher(a=pd_host, b=local_hostname).ratio()
+                logger.debug("Export locality compute score: Localhost: %s PD at: %s Score: %s"%(local_hostname, pd_host, pd_score))
+                if pd_score > max_score:
+                    best_pd=pd
+                    max_score=pd_score
+                
+                #pd_domain = tldextract.extract(pd.service_url).domain
+                #local_domain = tldextract.extract(socket.getfqdn()).domain
+                
+            if best_pd!=None:
+                logger.debug("Export from: %s"%(best_pd.service_url))
+                best_pd.export_du(self, target_url)
+                return
+                
             # No PD found. Utilize default PD
             logger.debug("Export from random PD")
             self.pilot_data[0].export_du(self, target_url)
@@ -643,21 +656,22 @@ class DataUnit(DataUnit):
     def _update_state(self, state):
         """ Internal method for updating state"""
         self.state=state
-        logger.debug("Update DU state to " + state + " Number of PD: " + str(len(self.pilot_data)))
-        if len(self.pilot_data) > 0: 
-            CoordinationAdaptor.update_du(self)
+        logger.debug("Update DU: "+ str(self.url) +  " state: " + state)
+        CoordinationAdaptor.update_du_state(self, state)
+        logger.debug("Updated DU: "+ str(self.url) +  " New state: " + self.get_state())
 
     
     def __add_pilot_data(self, pilot_data):
-        logger.debug("add du to pilot data")
-        
+        logger.debug("DU add_pilot_data: add DU to pilot data in Thread")
+        self._update_state(State.Pending)
         if len(self.pilot_data) > 0: # copy files from other pilot data
             self.pilot_data[0].copy_du(self, pilot_data)
         else: # copy files from original location
             pilot_data.put_du(self)
+        logger.debug("DU add_pilot_data: Copy/Put DU to pilot data successfull")    
         self.pilot_data.append(pilot_data)
         self._update_state(State.Running)
-        
+        logger.debug("DU add_pilot_data: Updated State")
         #self.url = CoordinationAdaptor.add_du(pilot_data.url, self)
         CoordinationAdaptor.update_du(self)
             
@@ -845,6 +859,58 @@ def test_data_unit_add_file():
     du_reconnect.add_files(["test.txt"])
     
     
+    
+class Lock(object):
+    def __init__(self, key, redis, expires=60, timeout=10):
+        """
+        Distributed locking using Redis SETNX and GETSET.
+
+        Usage::
+
+            with Lock('my_lock'):
+                print "Critical section"
+
+        :param  expires     We consider any existing lock older than
+                            ``expires`` seconds to be invalid in order to
+                            detect crashed clients. This value must be higher
+                            than it takes the critical section to execute.
+        :param  timeout     If another client has already obtained the lock,
+                            sleep for a maximum of ``timeout`` seconds before
+                            giving up. A value of 0 means we never wait.
+        """
+
+        self.key = key
+        self.timeout = timeout
+        self.expires = expires
+        self.redis = redis
+        
+    def __enter__(self):
+        timeout = self.timeout
+        while timeout >= 0:
+            expires = time.time() + self.expires + 1
+
+            if self.redis.setnx(self.key, expires):
+                # We gained the lock; enter critical section
+                return
+
+            current_value = self.redis.get(self.key)
+
+            # We found an expired lock and nobody raced us to replacing it
+            if current_value and float(current_value) < time.time() and \
+                self.redis.getset(self.key, expires) == current_value:
+                    return
+
+            timeout -= 1
+            time.sleep(1)
+
+        raise LockTimeout("Timeout whilst waiting for lock")
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        self.redis.delete(self.key)
+
+class LockTimeout(BaseException):
+    pass
+    
 
 if __name__ == "__main__":
     #test_data_unit_add_file()
